package mastodon

import (
	"fmt"
	"net/http"
	"net/url"
)

// List is metadata for a list of users.
type List struct {
	ID    ID     `json:"id"`
	Title string `json:"title"`
}

// GetLists returns all the lists on the current account.
func (c *Client) GetLists() ([]*List, error) {
	var lists []*List
	err := c.doAPI(http.MethodGet, "/api/v1/lists", nil, &lists, nil)
	if err != nil {
		return nil, err
	}
	return lists, nil
}

// GetAccountLists returns the lists containing a given account.
func (c *Client) GetAccountLists(id ID) ([]*List, error) {
	var lists []*List
	err := c.doAPI(http.MethodGet, fmt.Sprintf("/api/v1/accounts/%s/lists", url.PathEscape(string(id))), nil, &lists, nil)
	if err != nil {
		return nil, err
	}
	return lists, nil
}

// GetListAccounts returns the accounts in a given list.
func (c *Client) GetListAccounts(id ID) ([]*Account, error) {
	var accounts []*Account
	err := c.doAPI(http.MethodGet, fmt.Sprintf("/api/v1/lists/%s/accounts", url.PathEscape(string(id))), url.Values{"limit": {"0"}}, &accounts, nil)
	if err != nil {
		return nil, err
	}
	return accounts, nil
}

// GetList retrieves a list by ID.
func (c *Client) GetList(id ID) (*List, error) {
	var list List
	err := c.doAPI(http.MethodGet, fmt.Sprintf("/api/v1/lists/%s", url.PathEscape(string(id))), nil, &list, nil)
	if err != nil {
		return nil, err
	}
	return &list, nil
}

// CreateList creates a new list with a given title.
func (c *Client) CreateList(title string) (*List, error) {
	params := url.Values{}
	params.Set("title", title)

	var list List
	err := c.doAPI(http.MethodPost, "/api/v1/lists", params, &list, nil)
	if err != nil {
		return nil, err
	}
	return &list, nil
}

// RenameList assigns a new title to a list.
func (c *Client) RenameList(id ID, title string) (*List, error) {
	params := url.Values{}
	params.Set("title", title)

	var list List
	err := c.doAPI(http.MethodPut, fmt.Sprintf("/api/v1/lists/%s", url.PathEscape(string(id))), params, &list, nil)
	if err != nil {
		return nil, err
	}
	return &list, nil
}

// DeleteList removes a list.
func (c *Client) DeleteList(id ID) error {
	return c.doAPI(http.MethodDelete, fmt.Sprintf("/api/v1/lists/%s", url.PathEscape(string(id))), nil, nil, nil)
}

// AddToList adds accounts to a list.
//
// Only accounts already followed by the user can be added to a list.
func (c *Client) AddToList(list ID, accounts ...ID) error {
	params := url.Values{}
	for _, acct := range accounts {
		params.Add("account_ids", string(acct))
	}

	return c.doAPI(http.MethodPost, fmt.Sprintf("/api/v1/lists/%s/accounts", url.PathEscape(string(list))), params, nil, nil)
}

// RemoveFromList removes accounts from a list.
func (c *Client) RemoveFromList(list ID, accounts ...ID) error {
	params := url.Values{}
	for _, acct := range accounts {
		params.Add("account_ids", string(acct))
	}

	return c.doAPI(http.MethodDelete, fmt.Sprintf("/api/v1/lists/%s/accounts", url.PathEscape(string(list))), params, nil, nil)
}
